#[derive(Copy, Clone)]
pub struct Vertex {
    position: [f32; 4],
    color: [f32; 3]
}

fn vertex(pos: [i8; 3], color: [i8; 3]) -> Vertex {
    Vertex {
        position: [pos[0] as f32, pos[1] as f32, pos[2] as f32, 1.0],
        color: [color[0] as f32, color[1] as f32, color[2] as f32]
    }
}

pub fn create_vertices() -> (Vec<Vertex>, Vec<u16>) {
    let vertex_data = [
        // top (0, 0, 1)
        vertex([-1, -1, 1], [1, 0, 0]),
        vertex([1, -1, 1],  [1, 0, 0]),
        vertex([1, 1, 1],   [1, 0, 0]),
        vertex([-1, 1, 1],  [1, 0, 0]),
        // bottom (0, 0, -1)
        vertex([-1, 1, -1], [0, 1, 0]),
        vertex([1, 1, -1],  [0, 1, 0]),
        vertex([1, -1, -1], [0, 1, 0]),
        vertex([-1, -1, -1],[0, 1, 0]),
        // right (1, 0, 0)
        vertex([1, -1, -1], [0, 0, 1]),
        vertex([1, 1, -1],  [0, 0, 1]),
        vertex([1, 1, 1],   [0, 0, 1]),
        vertex([1, -1, 1],  [0, 0, 1]),
        // left (-1, 0, 0)
        vertex([-1, -1, 1],  [1, 1, 1]),
        vertex([-1, 1, 1],   [1, 1, 1]),
        vertex([-1, 1, -1],  [1, 1, 1]),
        vertex([-1, -1, -1], [1, 1, 1]),
        // front (0, 1, 0)
        vertex([1, 1, -1],  [1, 1, 0]),
        vertex([-1, 1, -1], [1, 1, 0]),
        vertex([-1, 1, 1],  [1, 1, 0]),
        vertex([1, 1, 1],   [1, 1, 0]),
        // back (0, -1, 0)
        vertex([1, -1, 1],   [0, 0, 0]),
        vertex([-1, -1, 1],  [0, 0, 0]),
        vertex([-1, -1, -1], [0, 0, 0]),
        vertex([1, -1, -1],  [0, 0, 0]),
    ];

    let index_data: &[u16] = &[
        0, 1, 2, 2, 3, 0, // top
        4, 5, 6, 6, 7, 4, // bottom
        8, 9, 10, 10, 11, 8, // right
        12, 13, 14, 14, 15, 12, // left
        16, 17, 18, 18, 19, 16, // front
        20, 21, 22, 22, 23, 20, // back
    ];

    (vertex_data.to_vec(), index_data.to_vec())
}


// OLD
/*
use std::mem::size_of;

const CUBE_VERTEX_SIZE: usize = size_of::<f32>() * 10;
const CUBE_VERTEX_COUNT: usize = 36;
const CUBE_VERTEX_POS_OFFSET: usize = 0;
const CUBE_VERTEX_COLOR_OFFSET: usize = size_of::<f32>() * 4;
const CUBE_VERTEX_UV_OFFSET: usize = size_of::<f32>() * 8;
const CUBE_VERTICES: [f32; 24] = [
    // float4 position, float4 color, float2 uv
    1, -1, 1, 1,   1, 0, 1, 1,  0, 1,
    -1, -1, 1, 1,  0, 0, 1, 1,  1, 1,
    -1, -1, -1, 1, 0, 0, 0, 1,  1, 0,
    1, -1, -1, 1,  1, 0, 0, 1,  0, 0,
    1, -1, 1, 1,   1, 0, 1, 1,  0, 1,
    -1, -1, -1, 1, 0, 0, 0, 1,  1, 0,

    1, 1, 1, 1,    1, 1, 1, 1,  0, 1,
    1, -1, 1, 1,   1, 0, 1, 1,  1, 1,
    1, -1, -1, 1,  1, 0, 0, 1,  1, 0,
    1, 1, -1, 1,   1, 1, 0, 1,  0, 0,
    1, 1, 1, 1,    1, 1, 1, 1,  0, 1,
    1, -1, -1, 1,  1, 0, 0, 1,  1, 0,

    -1, 1, 1, 1,   0, 1, 1, 1,  0, 1,
    1, 1, 1, 1,    1, 1, 1, 1,  1, 1,
    1, 1, -1, 1,   1, 1, 0, 1,  1, 0,
    -1, 1, -1, 1,  0, 1, 0, 1,  0, 0,
    -1, 1, 1, 1,   0, 1, 1, 1,  0, 1,
    1, 1, -1, 1,   1, 1, 0, 1,  1, 0,

    -1, -1, 1, 1,  0, 0, 1, 1,  0, 1,
    -1, 1, 1, 1,   0, 1, 1, 1,  1, 1,
    -1, 1, -1, 1,  0, 1, 0, 1,  1, 0,
    -1, -1, -1, 1, 0, 0, 0, 1,  0, 0,
    -1, -1, 1, 1,  0, 0, 1, 1,  0, 1,
    -1, 1, -1, 1,  0, 1, 0, 1,  1, 0,

    1, 1, 1, 1,    1, 1, 1, 1,  0, 1,
    -1, 1, 1, 1,   0, 1, 1, 1,  1, 1,
    -1, -1, 1, 1,  0, 0, 1, 1,  1, 0,
    -1, -1, 1, 1,  0, 0, 1, 1,  1, 0,
    1, -1, 1, 1,   1, 0, 1, 1,  0, 0,
    1, 1, 1, 1,    1, 1, 1, 1,  0, 1,

    1, -1, -1, 1,  1, 0, 0, 1,  0, 1,
    -1, -1, -1, 1, 0, 0, 0, 1,  1, 1,
    -1, 1, -1, 1,  0, 1, 0, 1,  1, 0,
    1, 1, -1, 1,   1, 1, 0, 1,  0, 0,
    1, -1, -1, 1,  1, 0, 0, 1,  0, 1,
    -1, 1, -1, 1,  0, 1, 0, 1,  1, 0,
];
*/